@@ -1,23 +1,24 @@
 from typing import Any, Callable, Iterable, List, Optional, Tuple, Union
 
 import eth_retry
+from cchecksum import to_checksum_address
 from eth_typing import Address, ChecksumAddress, HexAddress
 from eth_typing.abi import Decodable
-from eth_utils import to_checksum_address
 from web3 import Web3
 
-from multicall import Signature
 from multicall.constants import Network, w3
 from multicall.exceptions import StateOverrideNotSupported
 from multicall.loggers import setup_logger
-from multicall.utils import (chain_id, get_async_w3, run_in_subprocess,
-                             state_override_supported)
+from multicall.signature import Signature, _get_signature
+from multicall.utils import (_get_semaphore, chain_id, get_async_w3,
+                             run_in_subprocess, state_override_supported)
 
 logger = setup_logger(__name__)
 
 AnyAddress = Union[str,Address,ChecksumAddress,HexAddress]
 
 class Call:
+    __slots__ = "target", "returns", "block_id", "gas_limit", "state_override_code", "w3", "args", "function", "signature"
     def __init__(
         self, 
         target: AnyAddress, 
@@ -43,7 +44,7 @@ class Call:
             self.function = function
             self.args = None
 
-        self.signature = Signature(self.function)
+        self.signature = _get_signature(self.function)
     
     def __repr__(self) -> str:
         return f'<Call {self.function} on {self.target[:8]}>'
@@ -85,13 +86,13 @@ class Call:
             return decoded if len(decoded) > 1 else decoded[0]
 
     @eth_retry.auto_retry
-    def __call__(self, args: Optional[Any] = None, _w3: Optional[Web3] = None) -> Any:
+    def __call__(self, args: Optional[Any] = None, _w3: Optional[Web3] = None, *, block_id: Optional[int] = None) -> Any:
         _w3 = self.w3 or _w3 or w3
         args = prep_args(
             self.target,
             self.signature,
             args or self.args,
-            self.block_id,
+            block_id or self.block_id,
             self.gas_limit,
             self.state_override_code,
         )
@@ -100,25 +101,29 @@ class Call:
             self.signature,
             self.returns,
         )
+    
+    def __await__(self) -> Any:
+        return self.coroutine().__await__()
 
     @eth_retry.auto_retry
-    async def coroutine(self, args: Optional[Any] = None, _w3: Optional[Web3] = None) -> Any:
+    async def coroutine(self, args: Optional[Any] = None, _w3: Optional[Web3] = None, *, block_id: Optional[int] = None) -> Any:
         _w3 = self.w3 or _w3 or w3
 
         if self.state_override_code and not state_override_supported(_w3):
             raise StateOverrideNotSupported(f'State override is not supported on {Network(chain_id(_w3)).__repr__()[1:-1]}.')
         
-        args = await run_in_subprocess(
-            prep_args,
-            self.target,
-            self.signature,
-            args or self.args,
-            self.block_id,
-            self.gas_limit,
-            self.state_override_code,
-        )
-
-        output = await get_async_w3(_w3).eth.call(*args)
+        async with _get_semaphore():
+            output = await get_async_w3(_w3).eth.call(
+                *await run_in_subprocess(
+                    prep_args,
+                    self.target,
+                    self.signature,
+                    args or self.args,
+                    block_id or self.block_id,
+                    self.gas_limit,
+                    self.state_override_code,
+                )
+            )
 
         return await run_in_subprocess(Call.decode_output, output, self.signature, self.returns)
     
