@@ -1,7 +1,8 @@
+from functools import lru_cache
 from typing import Any, List, Optional, Tuple
 
 # For eth_abi versions < 2.2.0, `decode` and `encode` have not yet been added.
-# As we require web3 ^5.27, we require eth_abi compatability with eth_abi v2.0.0b6 and greater.
+# As we require web3 >=5.27, we require eth_abi compatability with eth_abi v2.0.0b6 and greater.
 try:
     from eth_abi import decode, encode
 except ImportError: 
@@ -11,6 +12,8 @@ from eth_typing.abi import Decodable, TypeStr
 from eth_utils import function_signature_to_4byte_selector
 
 
+get_4byte_selector = lru_cache(maxsize=None)(function_signature_to_4byte_selector)
+
 def parse_signature(signature: str) -> Tuple[str, List[TypeStr], List[TypeStr]]:
     """
     Breaks 'func(address)(uint256)' into ['func', ['address'], ['uint256']]
@@ -53,12 +56,19 @@ def parse_typestring(typestring: str) -> Optional[List[TypeStr]]:
     parts.append(part)
     return parts
 
+@lru_cache(maxsize=None)
+def _get_signature(signature: str) -> "Signature":
+    return Signature(signature)
 
 class Signature:
+    __slots__ = "signature", "function", "input_types", "output_types"
     def __init__(self, signature: str) -> None:
         self.signature = signature
         self.function, self.input_types, self.output_types = parse_signature(signature)
-        self.fourbyte = function_signature_to_4byte_selector(self.function)
+    
+    @property
+    def fourbyte(self) -> bytes:
+        return get_4byte_selector(self.function)
 
     def encode_data(self, args: Optional[Any] = None) -> bytes:
         return self.fourbyte + encode(self.input_types, args) if args else self.fourbyte
